﻿using System;
using System.Collections.Generic;

namespace TestModelForm
{
    
    public class BoardModel
    {
        enum Piece { White = 1, Black };
        private int[,] _board2DArray;
        private List<Direction> _directionList;
        private int _currentPlayer;
        public List<Square> legalMovesList { get; set; }
        public List<Square> piecesToFlipList { get; set; }
        private int _blackPieceCount = 0;
        private int _whitePieceCount = 0;

        public BoardModel()
        {
            _board2DArray = new int[8, 8];
            resetBoardArray();
            loadDirectionList();
        }

        public int getSquareValue(int x, int y)
        {
            return _board2DArray[x, y];
        }

        public void setSquareValue(int color, int x, int y)
        {
            _board2DArray[x, y] = color;
        }

        public int getCurrentPlayer()
        {
            return _currentPlayer;
        }

        public void setCurrentPlayer(int color)
        {
            _currentPlayer = color;
        }

        public int getWhitePieceCount()
        {
            return _whitePieceCount;
        }

        public int getBlackPieceCount()
        {
            return _blackPieceCount;
        }

        public bool loadLegalMoves(int color)
        { 
            // mehtod will load the legalMovesList with legal move Square objects.
            // method will also return true if there are legal moves for color, false if not.

            legalMovesList = new List<Square>();
            bool legalMoveExists = false;

            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    if (loadPiecesToFlip(color, x, y))
                    {
                        legalMoveExists = true;
                        Square temp = new Square(x, y);
                        legalMovesList.Add(temp);
                    }
                }
            }
            return legalMoveExists;
        }

        public bool loadPiecesToFlip(int color, int xStart , int yStart)
        {
            // mehtod loads piecesToFlipList and returns true if there is a legal move 
            // from xStart, yStart for given color, false if no valid move.

            bool isLegalMove = false;
            int opponent;
            piecesToFlipList = new List<Square>();

            // Return false if passed x,y is already occupied or not on the board.
            if (_board2DArray[xStart, yStart] > 0 || !isOnBoard(xStart, yStart)) { return false; }

            // set opponent for currentPlayer
            if (color == (int)Piece.Black) { opponent = (int)Piece.White; }
            else { opponent = (int)Piece.Black; }

            // look in all 8 directions for legal moves. if found add them too piecesToFlipList.
            foreach (Direction i in _directionList)
            {
                int x = xStart;
                int y = yStart;
                x += i.xDir;
                y += i.yDir;

                if (isOnBoard(x, y) && _board2DArray[x, y] == opponent)
                {
                    x += i.xDir;
                    y += i.yDir;
                    if(isOnBoard(x, y))
                    {
                        while (_board2DArray[x, y] == opponent)
                        {
                            x += i.xDir;
                            y += i.yDir;
                            if (!isOnBoard(x, y)) { break; }
                        }
                        if (isOnBoard(x, y))
                        {
                            if (_board2DArray[x, y] == color)
                            {
                                while (true)
                                {
                                    x -= i.xDir;
                                    y -= i.yDir;
                                    if (x == xStart && y == yStart) { break; }
                                    Square temp = new Square(x, y);
                                    piecesToFlipList.Add(temp);
                                    isLegalMove = true;
                                }
                            }
                        }
                    }

                }

            }
            return isLegalMove;
        }

        public void newGame()
        {
            // setup board2DArray and model for new game.
            resetBoardArray();
            _currentPlayer = (int)Piece.Black;
            _board2DArray[3, 3] = (int)Piece.White;
            _board2DArray[4, 3] = (int)Piece.Black;
            _board2DArray[3, 4] = (int)Piece.Black;
            _board2DArray[4, 4] = (int)Piece.White;
            loadPieceCount();
        }

        private void resetBoardArray()
        {
            // sets all elements in board2DArray to 0(empty).
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    _board2DArray[x,y] = 0;
                }
            }
        }

        private void loadDirectionList()
        {
            // loads direction list which will be used to look in all 8 directions for valid moves.
            _directionList = new List<Direction>();
            Direction north = new Direction(0, -1);
            Direction northEast = new Direction(1, -1);
            Direction east = new Direction(1, 0);
            Direction southEast = new Direction(1, 1);
            Direction south = new Direction(0, 1);
            Direction southWest = new Direction(-1, 1);
            Direction west = new Direction(-1, 0);
            Direction northWest = new Direction(-1, -1);
            _directionList.Add(north);
            _directionList.Add(northEast);
            _directionList.Add(east);
            _directionList.Add(southEast);
            _directionList.Add(south);
            _directionList.Add(southWest);
            _directionList.Add(west);
            _directionList.Add(northWest);
        }

        public void loadPieceCount()
        {
            // loads _blackPieceCount and _whitePieceCount
            _blackPieceCount = 0;
            _whitePieceCount = 0;

            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    if (_board2DArray[x, y] == (int)Piece.Black) { _blackPieceCount++; }
                    else if (_board2DArray[x, y] == (int)Piece.White) { _whitePieceCount++; }
                }
            }
        }

        private bool isOnBoard(int x, int y)
        {
            // returns true if x,y coordinate is on the board.
            return (x >= 0 && x <= 7 && y >= 0 && y <= 7);
        }

    }
}
